/******************************************************************************

  driver.c

  The list of all available drivers. Drivers have to be included here to be
  recognized by the executable.

  To save some typing, we use a hack here. This file is recursively #included
  twice, with different definitions of the DRIVER() macro. The first one
  declares external references to the drivers; the second one builds an array
  storing all the drivers.

******************************************************************************/

#include "driver.h"


#ifndef DRIVER_RECURSIVE

/* The "root" driver, defined so we can have &driver_##NAME in macros. */
struct GameDriver driver_0 =
{
	__FILE__,
	0,
	"",
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	NOT_A_DRIVER
};

#endif

#ifdef TINY_COMPILE

extern struct GameDriver TINY_NAME;

const struct GameDriver *drivers[] =
{
	TINY_POINTER,
	0	/* end of array */
};

#else

#ifndef DRIVER_RECURSIVE

#define DRIVER_RECURSIVE

/* step 1: declare all external references */
#define DRIVER(NAME) extern struct GameDriver driver_##NAME;
#define TESTDRIVER(NAME) extern struct GameDriver driver_##NAME;
#include "driver.c"

/* step 2: define the drivers[] array */
#undef DRIVER
#undef TESTDRIVER
#define DRIVER(NAME) &driver_##NAME,
#define TESTDRIVER(NAME)
const struct GameDriver *drivers[] =
{
#include "driver.c"
	0	/* end of array */
};

#else	/* DRIVER_RECURSIVE */

//DRIVERNV(m_mpac  )      //BY35-872: 05/82 Mr. & Mrs. Pac-Man
//DRIVERNV(babypac )      //??        10/82 Baby Pac-Man

DRIVER( babypac )
DRIVER( m_mpac )

/* PacMAME */
//DRIVER( mspac4kp ) /*BlueJustice*/
//DRIVER( mspaccp ) /*BlueJustice*/
//DRIVER( pacman1k ) /*BlueJustice*/
//DRIVER( pacman2k ) /*BlueJustice*/
//DRIVER( pacman3k ) /*BlueJustice*/
//DRIVER( pacmanb2 ) /*BlueJustice*/
//DRIVER( pacmanb3 ) /*BlueJustice*/
//DRIVER( pacmanb6 ) /*BlueJustice*/
//DRIVER( pacmanbm ) /*BlueJustice*/

DRIVER( 20pacgal )
DRIVER ( 4in1 )      /* Armenia 1981? */
DRIVER( abortman )
DRIVER( baby2 )
DRIVER( baby3 )
DRIVER( baby4 )
DRIVER( bjhang1k ) /*BlueJustice*/
DRIVER( bjhang2k ) /*BlueJustice*/
DRIVER( bjhangb ) /*BlueJustice*/
DRIVER( bjhangb5 ) /*BlueJustice*/
DRIVER( bjhangb7 ) /*BlueJustice*/
DRIVER( bjhangbp ) /*BlueJustice*/
DRIVER( bjhanggp ) /*BlueJustice*/
DRIVER( bjhangp ) /*BlueJustice*/
DRIVER( bjjr1k ) /*BlueJustice*/
DRIVER( bjjr2k ) /*BlueJustice*/
DRIVER( bjjr2k1 ) /*BlueJustice*/
DRIVER( bjjr2k1p ) /*BlueJustice*/
DRIVER( bjjr2k2 ) /*BlueJustice*/
DRIVER( bjjr2k2p ) /*BlueJustice*/
DRIVER( bjjr2k3 ) /*BlueJustice*/
DRIVER( bjjr2k3p ) /*BlueJustice*/
DRIVER( bjjr2k4 ) /*BlueJustice*/
DRIVER( bjjr2k4p ) /*BlueJustice*/
DRIVER( bjjr2k5 ) /*BlueJustice*/
DRIVER( bjjr2k5p ) /*BlueJustice*/
DRIVER( bjjr3kp ) /*BlueJustice*/
DRIVER( bjjr4kp ) /*BlueJustice*/
DRIVER( bjjr5kp ) /*BlueJustice*/
DRIVER( bjjr6kp ) /*BlueJustice*/
DRIVER( bjjr7k ) /*BlueJustice*/
DRIVER( bjjr7kp ) /*BlueJustice*/
DRIVER( bjjr8k ) /*BlueJustice*/
DRIVER( bjjr8kp ) /*BlueJustice*/
DRIVER( bjjr9kp ) /*BlueJustice*/
DRIVER( bjjrdx ) /*BlueJustice*/
DRIVER( bjjrjr ) /*BlueJustice*/
DRIVER( bjjrjrp ) /*BlueJustice*/
DRIVER( bjjrp ) /*BlueJustice*/
DRIVER( brakman )
DRIVER( bucaneer )
DRIVER( caterpil )
DRIVER( chthang )
DRIVER( chtmsatk )
DRIVER( chtmspa )
DRIVER( chtpac )
DRIVER( chtpman2 )
DRIVER( chtpop )
DRIVER( chtpuck )
DRIVER( cookiem )
DRIVER( crash )
DRIVER( crazypac )
DRIVER( dizzy )
DRIVER( eltonpac )
DRIVER( europac )
DRIVER( fasthang )
DRIVER( fasthear )
DRIVER( fastjr )
DRIVER( fastmspa )
DRIVER( fastpac )
DRIVER( fastplus )
DRIVER( fastpop )
DRIVER( fastpuck )
DRIVER( faststrm )
DRIVER( fpnleash )
DRIVER( fstmsatk )
DRIVER( fstpman2 )
DRIVER( ghostmun )	/*MAME*/ /* bootleg */
DRIVER( gmunch )
DRIVER( gmunch2 )
DRIVER( hangly )	/*MAME*/ /* hack */
DRIVER( hangly2 )	/*MAME*/ /* hack */
DRIVER( hangly2x )
DRIVER( hanglyad )
DRIVER( hanglyb )
DRIVER( hanglyx )
DRIVER( heartbn2 )
DRIVER( heartbrn )
DRIVER( hearts )
DRIVER( jrhearts )
DRIVER( jrpacad )
DRIVER( jrpacman )	/*MAME*/ /* (c) 1983 Midway */
DRIVER( jrvectr )
DRIVER( mariopac )
DRIVER( mazeman )
DRIVER( mrpacman )
DRIVER( ms2600 )
DRIVER( msatk2ad )
DRIVER( msatkad )
DRIVER( msbaby )
DRIVER( msbaby1 )
DRIVER( msberzk )
DRIVER( mschamp )	/*MAME*/ /* hack */
DRIVER( msdroid )
DRIVER( msdstorm )
DRIVER( mselton )
DRIVER( msf1pac )
DRIVER( msgrkatk )
DRIVER( msgrkb )
DRIVER( mshangly )
DRIVER( mshearts )
DRIVER( msindy )
DRIVER( msmini )
DRIVER( msminia )
DRIVER( msnes4a )
DRIVER( msnes62 )
DRIVER( msnes63 )
DRIVER( msnes6m )
DRIVER( msnes6m2 )
DRIVER( msnes6m3 )
DRIVER( msnes6m4 )
DRIVER( mspac1k ) /*BlueJustice*/
DRIVER( mspac2k ) /*BlueJustice*/
DRIVER( mspac3k ) /*BlueJustice*/
DRIVER( mspac5k ) /*BlueJustice*/
DRIVER( mspac5kp ) /*BlueJustice*/
DRIVER( mspac6m )
DRIVER( mspacad )
DRIVER( mspacat2 )
DRIVER( mspacatb )
DRIVER( mspaccm ) /*BlueJustice*/
DRIVER( mspacdx ) /*BlueJustice*/
DRIVER( mspacex ) /*BlueJustice*/
DRIVER( mspacexp ) /*BlueJustice*/
DRIVER( mspachi )
DRIVER( mspacii )
DRIVER( mspacj )
DRIVER( mspacmab )	/*MAME*/ /* bootleg */
DRIVER( mspacman )	/*MAME*/ /* (c) 1981 Midway */	/* made by Gencomp */
DRIVER( mspacmat )	/*MAME*/ /* hack */
DRIVER( mspacpls )	/*MAME*/ /* hack */
DRIVER( mspacren )
DRIVER( mspacrip )
DRIVER( mspacsm ) /*BlueJustice*/
DRIVER( mspacx )
DRIVER( mspatkx )
DRIVER( mspc6mad )
DRIVER( msrumble )
DRIVER( mssilad )
DRIVER( msultra )
DRIVER( msvctr6m )
DRIVER( msvectr )
DRIVER( msyakman )
DRIVER( multi10 )
DRIVER( multi11 )
DRIVER( multi13 )
DRIVER( multi14 )
DRIVER( multi15 )
DRIVER( newpir )
DRIVER( newpirb )
DRIVER( ncv2 )		/*MAME*/ /* (c) 1996 */
DRIVER( ncv2j )		/*MAME*/ /* (c) 1996 (Japan) */
DRIVER( newpuckx )	/*MAME*/ /* hack */
DRIVER( newpuc2 )
DRIVER( nwpuc2b )
/*
DRIVER( npacmod )
*/
DRIVER( npacmodx )
DRIVER( pac2600 )
DRIVER( pacatk )
DRIVER( pacbaby )
DRIVER( pacbaby2 )
DRIVER( pacbaby3 )
DRIVER( pacbell )
DRIVER( pacelec )
DRIVER( pacgal )	/*MAME*/ /* hack */
DRIVER( pacgalu )
DRIVER( pacgalx )
DRIVER( pacheart )	/*MAME*/ /* hack */
DRIVER( pacinvis )
DRIVER( pacjail )
DRIVER( pacjr1 )
DRIVER( pacjr2 )
DRIVER( pacjr3 )
DRIVER( pacjr4 )
DRIVER( pacland )	/*MAME*/ /* (c) 1984 */
DRIVER( pacland2 )	/*MAME*/ /* (c) 1984 */
DRIVER( pacland3 )	/*MAME*/ /* (c) 1984 */
DRIVER( paclandm )	/*MAME*/ /* (c) 1984 Midway */
DRIVER( paclandb )
DRIVER( paclnddx ) /*BlueJustice*/
DRIVER( pacm255 )
DRIVER( pacman )	/*MAME*/ /* (c) 1980 Midway */
DRIVER( pacman3d )
DRIVER( pacman4k ) /*BlueJustice*/
DRIVER( pacman5k ) /*BlueJustice*/
DRIVER( pacman6 )
DRIVER( pacman6k ) /*BlueJustice*/
DRIVER( pacman7k ) /*BlueJustice*/
DRIVER( pacman7p ) /*BlueJustice*/
DRIVER( pacmanb2 ) /*BlueJustice*/
DRIVER( pacmanb3 ) /*BlueJustice*/
DRIVER( pacmanb4 ) /*BlueJustice*/
DRIVER( pacmanb6 ) /*BlueJustice*/
DRIVER( pacmanb8 ) /*BlueJustice*/
DRIVER( pacmanbl )	/*MAME*/ /* bootleg */
DRIVER( pacmanbp ) /*BlueJustice*/
DRIVER( pacmandx ) /*BlueJustice*/
DRIVER( pacmanex ) /*BlueJustice*/
DRIVER( pacmanfp ) /*BlueJustice*/
DRIVER( pacmanfv ) /*BlueJustice*/
DRIVER( pacmania )	/*MAME*/ /* (c) 1987 */
DRIVER( pacmanij )	/*MAME*/ /* (c) 1987 (Japan) */
/*
DRIVER( pacmanjp )
DRIVER( pacmanm )
*/
DRIVER( pacmanx )
DRIVER( pacmanxj )
DRIVER( pacmanxm )
DRIVER( pacmar )
DRIVER( pacmbp ) /*BlueJustice*/
DRIVER( pacmini )
DRIVER( pacmini2 )
DRIVER( pacmn6m2 )
DRIVER( pacmod )	/*MAME*/ /* (c) 1981 Midway */
DRIVER( pacmodx )
DRIVER( pacms1 )
DRIVER( pacms2 )
DRIVER( pacms3 )
DRIVER( pacms4 )
DRIVER( pacmsa1 )
DRIVER( pacmsa2 )
DRIVER( pacmsa3 )
DRIVER( pacmsa4 )
/*
DRIVER( pacmulti )
*/
DRIVER( pacn255 )
DRIVER( pacnchmp )	/*MAME*/ /* (c) 1983 */
DRIVER( pacnpal )	/*MAME*/ /* (c) 1983 */
DRIVER( pacnpal2 )	/*MAME*/ /* (c) 1983 */
DRIVER( pacplus )	/*MAME*/
DRIVER( pacplusc )
DRIVER( pacplusx )
DRIVER( pacshuf )
DRIVER( pacspeed )
DRIVER( pacstrm )
DRIVER( pacweird )
DRIVER( pcrunchy )
DRIVER( pengman )
DRIVER( petshop )
DRIVER( pheartb )
DRIVER( pheartx )
DRIVER( piranha )	/*MAME*/ /* hack */
DRIVER( plusfast )
DRIVER( pmad )
DRIVER( pmad00 )
DRIVER( pmad6m )
DRIVER( pmada )
DRIVER( pmanalt )
DRIVER( pmanaltm )
DRIVER( pnc )
DRIVER( pplusad )
DRIVER( profpac )	/*MAME*/ /* (c) 1983 Bally Midway */
DRIVER( puckman )	/*MAME*/ /* (c) 1980 Namco */
DRIVER( puckman2 )
DRIVER( puckmana )	/*MAME*/ /* (c) 1980 Namco */
DRIVER( puckmanx )
DRIVER( puckmod )	/*MAME*/ /* (c) 1981 Namco */
DRIVER( puckren )
DRIVER( puckrenc )
DRIVER( punleash )
DRIVER( roboman )
DRIVER( snakeyes )
DRIVER( snowpac )
DRIVER( spacplus ) /*BlueJustice*/
DRIVER( sueworld )
DRIVER( sumelton )
DRIVER( superabc )
DRIVER( superpac )	/*MAME*/ /* (c) 1982 */
DRIVER( superpcm )	/*MAME*/ /* (c) 1982 Midway */
DRIVER( superpcn )
DRIVER( tbone )
DRIVER( test )
DRIVER( ultra2 )
DRIVER( ultrapac )
DRIVER( vcrunchy )
DRIVER( vecbaby )
DRIVER( vecbaby2 )
DRIVER( vecbaby3 )
DRIVER( vectplus )
DRIVER( vectr6m )
DRIVER( vectr6tb )
DRIVER( vectratk )
DRIVER( vectrpac )
DRIVER( vectrsup )
DRIVER( vectxens )
DRIVER( vhangly )
DRIVER( vpacbell )
DRIVER( vpacelec )
DRIVER( vpacjail )
DRIVER( vpacms1 )
DRIVER( vpacms2 )
DRIVER( vpacms3 )
DRIVER( vpacms4 )
DRIVER( vpacmsa1 )
DRIVER( vpacmsa2 )
DRIVER( vpacmsa3 )
DRIVER( vpacmsa4 )
DRIVER( vpacshuf )
DRIVER( vpspeed )
DRIVER( xensad )
DRIVER( xensrev )
DRIVER( zolaad )
DRIVER( zolaatk )
DRIVER( zolamaze )
DRIVER( zolapac )
DRIVER( zolaslow )
DRIVER( zolatunl )
DRIVER( zolavect )
TESTDRIVER( babypac )
TESTDRIVER( hyperpac )
TESTDRIVER( hyperpcb )
TESTDRIVER( jrfast )

/* End PacMAME */

#endif	/* DRIVER_RECURSIVE */

#endif	/* TINY_COMPILE */
